@@ -17,7 +17,7 @@
 
 package Git::Hooks::CheckLog;
 {
-  $Git::Hooks::CheckLog::VERSION = '0.036';
+  $Git::Hooks::CheckLog::VERSION = '0.037';
 }
 # ABSTRACT: Git::Hooks plugin to enforce commit log policies.
 
@@ -87,10 +87,10 @@ sub _spell_checker {
     }
 
     unless (state $tried_to_check) {
-        eval { require Text::SpellChecker; };
-        length $@
-            and $git->error($PKG, "Could not require Text::SpellChecker module to spell messages.\n$@\n")
-                and return;
+        unless (eval { require Text::SpellChecker; }) {
+            $git->error($PKG, "Could not require Text::SpellChecker module to spell messages.\n$@\n");
+            return;
+        }
 
         # Text::SpellChecker uses either Text::Hunspell or
         # Text::Aspell to perform the checks. But it doesn't try to
@@ -307,7 +307,7 @@ Git::Hooks::CheckLog - Git::Hooks plugin to enforce commit log policies.
 
 =head1 VERSION
 
-version 0.036
+version 0.037
 
 =head1 DESCRIPTION
 
