@@ -17,7 +17,7 @@
 
 package Git::Hooks::GerritChangeId;
 {
-  $Git::Hooks::GerritChangeId::VERSION = '0.036';
+  $Git::Hooks::GerritChangeId::VERSION = '0.037';
 }
 # ABSTRACT: Git::Hooks plugin to insert a Change-Id in a commit message.
 
@@ -55,8 +55,8 @@ sub clean_message {
 
     return '' unless length $msg;
 
-    # remove empty lines from the end
-    $msg =~ s/\n{2,}$/\n/;
+    # remove empty lines from the end, leaving a \n there
+    $msg =~ s/(?<=\n)\n+$//;
 
     return $msg;
 }
@@ -73,7 +73,7 @@ sub gen_change_id {
         [ committer => [qw/var GIT_COMMITTER_IDENT/] ],
     ) {
         try {
-            $fh->print($info->[0], ' ', $git->command($info->[1], {STDERR => 0}), "\n");
+            $fh->print($info->[0], ' ', scalar($git->command($info->[1], {STDERR => 0})));
         } otherwise {
             # Can't find info. That's ok.
         };
@@ -185,7 +185,7 @@ Git::Hooks::GerritChangeId - Git::Hooks plugin to insert a Change-Id in a commit
 
 =head1 VERSION
 
-version 0.036
+version 0.037
 
 =head1 DESCRIPTION
 
